----------------------------------------------------------------------
-- CruiseControl v0.9
-- Author: Aryl (Aryl)
-- last change by Aryl on 2011-02-26T01:00:13Z

----------------------------------------------------------------------
-- print routines
--
local HEADER = "|cFFccb938CruiseControl:|r ";
local function msg(s)
      DEFAULT_CHAT_FRAME:AddMessage(HEADER..s);
end
local function debug(s)
      --[===[@debug@
      DEFAULT_CHAT_FRAME:AddMessage(HEADER..s);
      --@end-debug@]===]
end

local function error(s)
      DEFAULT_CHAT_FRAME:AddMessage(HEADER.."|cFFcf672fERROR:|r "..s);
end

local function say_engaged()
      DEFAULT_CHAT_FRAME:AddMessage(HEADER.."|cFFb0d032engaged.|r");
end
local function say_disengaged()
      DEFAULT_CHAT_FRAME:AddMessage(HEADER.."|cFF7091f2disengaged.|r");
end
local function say_user_disengaged()
      DEFAULT_CHAT_FRAME:AddMessage(HEADER.."|cFF7091f2user disengaged.|r");
end
local function say_approaching()
      DEFAULT_CHAT_FRAME:AddMessage(HEADER.."approaching destination...");
end
local function say_reached()
      DEFAULT_CHAT_FRAME:AddMessage(HEADER.."|cFFdf772fdestination reached.|r");
end

debug("loading...");

-------------------------------------------------------------------------
-- main frame and libs
--
CruiseControl = CreateFrame("Frame", nil, WorldMapDetailFrame);

local Astrolabe = DongleStub("Astrolabe-1.0");

----------------------------------------------------------------------
-- config vars
local UPDATE_INTERVAL = 0.175; -- seconds
local SOUND_INTERVAL = 1; -- seconds
local APPROACH_LIMIT = 11; -- seconds
local LINEWIDTH = 48;
local LINECOLOR = {0.5, 0.7, 0.8, 0.7};
local RINGWIDTH = 24;
local RINGCOLOR = {0.9, 0.9, 0.9, 1};
local FONTSIZE = 12;
local FONTSTYLE = "Fonts\\FRIZQT__.TTF";
local FONTOUTLINE = "OUTLINE";
local TARGETICONFILE = "Interface\\AddOns\\CruiseControl\\ring.tga";

----------------------------------------------------------------------
-- sound settings

local SOUNDSETTINGS = {
  ["Sound_MasterVolume"] = 1.0,
  ["Sound_SFXVolume"] = 0.0,
  ["Sound_MusicVolume"] = 0.0,
  ["Sound_AmbienceVolume"] = 0.0,
  ["Sound_EnableSoundWhenGameIsInBG"] = 1.0,
};

----------------------------------------------------------------------
-- sound styles
local SOUNDSTYLES = {
  ["alt"] = {  
    ["DESTSET"]    = "Interface\\AddOns\\CruiseControl\\click.mp3",
    ["DESTCLEAR"]  = "Interface\\AddOns\\CruiseControl\\cluck.mp3",
    ["DESTCLEAR2"] = "Interface\\AddOns\\CruiseControl\\knack.mp3",
    ["APPROACH"]   = "Interface\\AddOns\\CruiseControl\\beep.mp3",
    ["OVERSHOT"]   = "Interface\\AddOns\\CruiseControl\\piu.mp3",
  },
  ["default"] = {
    ["DESTSET"]    = "Sound\\INTERFACE\\PickUp\\PutDownRing.ogg",
    ["DESTCLEAR"]  = "Sound\\INTERFACE\\uEscapeScreenClose.ogg", 
    ["DESTCLEAR2"] = "Sound\\Doodad\\DropOnGround.ogg",
    ["APPROACH"]   = "Sound\\INTERFACE\\AuctionWindowOpen.ogg", 
    ["OVERSHOT"]   = "Sound\\Creature\\Monkey\\MonkeyWound_01.ogg",
  },
  ["chicken"] = {
    ["DESTSET"]    = "Sound\\Creature\\BushChicken\\BushChicken_Wound_02.ogg",
    ["DESTCLEAR"]  = "Sound\\Creature\\BushChicken\\BushChicken_Wound_09.ogg",
    ["DESTCLEAR2"] = "Sound\\Creature\\BushChicken\\BushChicken_PreAggro_07.ogg",
    ["APPROACH"]   = "Sound\\Creature\\BushChicken\\BushChicken_Attack_10.ogg",
    ["OVERSHOT"]   = "Sound\\Creature\\ChickenMount\\ChickenMountWound2.ogg",
  },
  ["animals"] = {
    ["DESTSET"]    = "Sound\\Creature\\Monkey\\MonkeyAttack_03.ogg",
    ["DESTCLEAR2"] = "Sound\\Creature\\Gorilla\\GorillaAttackA.ogg",
    ["DESTCLEAR"]  = "Sound\\Creature\\Seagull\\Seagull_PreAggro_04.ogg",
    ["APPROACH"]   = "Sound\\Creature\\PugDog\\PugDog_Clickable_06.ogg",
    ["OVERSHOT"]   = "Sound\\Creature\\Seagull\\Seagull_Wound_07.ogg",
  },
};

local function playMySound(type)
  PlaySoundFile(SOUNDSTYLES[CruiseControlOptions.SoundStyle][type], "Master");
end

--    ["DESTCLEAR2"] = "Sound\\Doodad\\DR_MachineParts01_Open.ogg", -- DropOnGround.ogg";
--local DESTSET   = "Sound\\INTERFACE\\uMiniMapZoom.ogg";
--local APPROACH  = "Sound\\Item\\Weapons\\ParrySounds\\1hParryMetalHitMetal.ogg";
--local APPROACH  = "Sound\\Spells\\Tradeskills\\BlackSmithCraftingD.ogg";
--local OVERSHOT  = "Sound\\Creature\\MobileAlertBot\\MobileAlertBotWoundA.ogg"; 
--local OVERSHOT  = "Sound\\Creature\\Monkey\\Monkey_Wound_07.ogg"; 
--local OVERSHOT  = "Sound\\Creature\\Parrot\\PArrotPreAggro.ogg";
--    ["DESTCLEAR2"] = "Sound\\Creature\\Parrot\\PArrotAggro.ogg";
--local OVERSHOT  = "Sound\\Creature\\PugDog\\PugDog_Clickable_06";
--local OVERSHOT  = "Sound\\Creature\\Seagull\\Seagull_Wound_07.ogg";
--local OVERSHOT  = "Sound\\Creature\\Seagull\\Seagull_Wound_04.ogg";
--local OVERSHOT  = "Sound\\INTERFACE\\RaidWarning.ogg";

--local SOUND = {
--      "Sound\\Doodad\\ArcaneCrystalOpen.ogg", -- crash
--      "Sound\\Doodad\\Deepholm_MercuryDrip01_01.ogg",
--      "Sound\\Doodad\\DR_MachineParts01_Open.ogg",
--      "Sound\\Doodad\\G_BarrelOpen-Chest1.ogg",
--      "Sound\\Doodad\\G_HunterTrapSpawn.ogg",
--      "Sound\\Doodad\\GnomeElevatorSmallStop.ogg",
--      "Sound\\Doodad\\GO_DeepholmFloatingRock_Whoosh_01.ogg",
--      "Sound\\Doodad\\GO_MummyHand01_04.ogg",
--      "Sound\\Doodad\\GO_MummyHand01_06.ogg",
--      "Sound\\Doodad\\Goblin_KezanBillboard_01_Lights_01.ogg",
--      "Sound\\INTERFACE\\uMiniMapZoom.ogg",
--      "Sound\\INTERFACE\\SheathMetal.ogg",
--      "Sound\\INTERFACE\\igQuestFailed.ogg",
--      "Sound\\INTERFACE\\DropOnGround.ogg",
--      "Sound\\INTERFACE\\RaidWarning.ogg",
--      "Sound\\INTERFACE\\AuctionWindowOpen.ogg", -- ding
--      "Sound\\INTERFACE\\PVPFlagTakenMono.ogg",
--      "Sound\\INTERFACE\\UI_BnetToast.ogg",  -- uinggg
--      "Sound\\INTERFACE\\AuctionWindowClose.ogg",
--      "Sound\\INTERFACE\\LFG_Denied.ogg",
--      "Sound\\INTERFACE\\PutDownRing.ogg",
--      "Sound\\Item\\1hParryMetalHitMetal.ogg",
--};

----------------------------------------------------------------------
-- slash command handling
SLASH_CruiseControl1 = "/CruiseControl";
SLASH_CruiseControl2 = "/CC";
function SlashCmdList.CruiseControl(arg, editbox)
  local command, rest = arg:match("^(%S*)%s*(.-)$");
  if command == "soundstyle" then
    if SOUNDSTYLES[rest] then
      CruiseControlOptions.SoundStyle = rest;
      msg("soundstyle set to "..rest);
    else
      msg("unknown soundstyle: "..rest);
    end
  else
    msg("Syntax: /CruiseControl soundstyle <style>");
  end
end

----------------------------------------------------------------------
-- global vars
local PlayerX = 0;
local PlayerY = 0;
local Angle = 0;
local MouseX = nil;
local MouseY = nil;

local Prev = {};
Prev.C, Prev.Z, Prev.X, Prev.Y = Astrolabe:GetCurrentPlayerPosition();
local MaxSpeed = 7;
local From = {};
From.TOA = 0;

--------------------------------------------------------------------
-- GetCurrentCursorPosition
--
local function GetCurrentCursorPosition()
	local x, y = GetCursorPosition()
	local left, top = WorldMapDetailFrame:GetLeft(), WorldMapDetailFrame:GetTop()
	local width = WorldMapDetailFrame:GetWidth()
	local height = WorldMapDetailFrame:GetHeight()
	local scale = WorldMapDetailFrame:GetEffectiveScale()
	local cx = (x/scale - left) / width
	local cy = (top - y/scale) / height

	if cx < 0 or cx > 1 or cy < 0 or cy > 1 then
		return nil, nil
	end
	return cx, cy
end



-------------------------------------------------------------------
-- Init
--
function CruiseControl:Init()
  self.timeSinceLastUpdate = 0;
  self.timeSinceLastSound = 0;
  self.currentUnit = "player";
  self.targetSet = false;
  self.startedFlying = false;
  self.targetReached = false;
  self.approachAnnounced = false;

  if CruiseControlOptions == nil then
    CruiseControlOptions = {};
    CruiseControlOptions.SoundStyle = 'default';
    debug("setting sound to default");
  else
    if not SOUNDSTYLES[CruiseControlOptions.SoundStyle] then
      CruiseControlOptions.SoundStyle = "default";
      debug("resetting sound to default");
    end
    debug("sound style is "..CruiseControlOptions.SoundStyle);
  end

  -- text display on world map
  self.textString = WorldMapButton:CreateFontString(nil, "OVERLAY");
  self.textString:SetTextColor(1, 1, 1, 1);
  self.textString:SetPoint("BOTTOMLEFT", WorldMapButton, "BOTTOMLEFT", 3, 3);

  self:SetWidth(self:GetParent():GetWidth());
  self:SetHeight(self:GetParent():GetHeight());
  self:SetPoint("TOPLEFT");
  self:EnableMouse(true);
  self:Show();

  -- 
  self.approachFrame = CreateFrame("Frame", "CC_Approach", UIParent);
  self.approachFrame.timeSinceLastSound = 0;
  self.approachFrame:SetScript("OnUpdate", function (self, elapsed)
    self.timeSinceLastSound = self.timeSinceLastSound + elapsed;
    if self.timeSinceLastSound < SOUND_INTERVAL then
      return;
    end
    local tsls = self.timeSinceLastSound;
    self.timeSinceLastSound = 0;
    if not CruiseControl.targetSet then
      Prev.C, Prev.Z, Prev.X, Prev.Y = Astrolabe:GetCurrentPlayerPosition();
      --Prev.takenAt = GetTime();
      return;
    end
    local c, z, x, y = Astrolabe:GetCurrentPlayerPosition();
    local dist = Astrolabe:ComputeDistance(Prev.C, Prev.Z, Prev.X, Prev.Y, c, z, x, y);
    --local diff = GetTime() - Prev.takenAt;
    local speed = 0;
    if dist then
      speed = dist / tsls;
    end
    if speed > 0 then
      CruiseControl.startedFlying = true;
    else
      -- we stopped, disengage
      if CruiseControl.startedFlying then
        say_disengaged();
        if CruiseControl.targetReached then
          playMySound("DESTCLEAR2");
	else
          playMySound("DESTCLEAR");
	end
        if CruiseControl.soundSettings then
	  for k, v in pairs(CruiseControl.soundSettings) do
	    SetCVar(k, v);
	  end
 	end
	CruiseControl.soundSettings = nil;
        CruiseControl.targetSet = false;	
        CruiseControl.startedFlying = false;
	CruiseControl.targetReached = false;
	CruiseControl.approachAnnounced = false;
      end
    end
    Prev.C, Prev.Z, Prev.X, Prev.Y = c, z, x, y; --Astrolabe:GetCurrentPlayerPosition();
    Prev.takenAt = GetTime();
    if CruiseControl.targetSet then
      dist = From.Distance - Astrolabe:ComputeDistance(From.C, From.Z, From.X, From.Y, c, z, x, y);
      if speed > 0 then
        From.TOA = dist / speed;
      else
        From.TOA = 0;
      end
      if dist < speed * SOUND_INTERVAL then
        if not CruiseControl.targetReached then
          say_reached();
          CruiseControl.targetReached = true;
        end
	playMySound("OVERSHOT");
      else
        if dist < speed * APPROACH_LIMIT then
	  if not CruiseControl.approachAnnounced then
            say_approaching();
	    CruiseControl.approachAnnounced = true;
	    -- save sound settings and set to loud
	    CruiseControl.soundSettings = {};
	    for k, v in pairs(SOUNDSETTINGS) do
	      CruiseControl.soundSettings[k] = GetCVar(k);
	      SetCVar(k, v);
	    end
	  end
	  playMySound("APPROACH");
	end
      end
    end
  end
  );

  self.target = CreateFrame("Frame", "CC_Target", self);
  self.target:SetSize(RINGWIDTH, RINGWIDTH);
  self.targetTexture = self.target:CreateTexture("CC_TargetTexture", "OVERLAY");
  if not self.targetTexture:SetTexture(TARGETICONFILE) then
    error("Cannot load ring.tga");
  end;
  self.targetTexture:SetParent(self.target);
  self.targetTexture:SetAllPoints();
  self.targetTexture:SetSize(RINGWIDTH, RINGWIDTH);
  self.targetTexture:SetVertexColor(RINGCOLOR[0], RINGCOLOR[1], RINGCOLOR[2], RINGCOLOR[3]);
  self.targetTexture:Show();
  self.target:SetFrameStrata("HIGH");
  self.target:EnableMouse(true);
  self.target:SetScript("OnMouseUp", function (self, buttonName)
    --debug("clicked: "..buttonName);
    if (buttonName == "LeftButton") then
      -- remember where we are and how far we want to go.
      From.C, From.Z, From.X, From.Y = Astrolabe:GetCurrentPlayerPosition();
      local mx, my = GetCurrentCursorPosition();
      From.Distance = Astrolabe:ComputeDistance(From.C, From.Z, From.X, From.Y, From.C, From.Z, mx, my);
--      if CruiseControl.targetSet then
--        CruiseControl.targetSet = false;
--        say_disengaged();
--        playMySound("DESTCLEAR");
--      else
        CruiseControl.targetSet = true;
        CruiseControl.startedFlying = false;
	CruiseControl.approachAnnounced = false;
	CruiseControl.targetReached = false;
        say_engaged();
        playMySound("DESTSET");
--      end
    end
    if (buttonName == "RightButton") then
      CruiseControl.targetSet = false;
      say_user_disengaged();
      playMySound("DESTCLEAR");
    end
  end);

  msg("initialized.");
end

----------------------------------------------------------------------
-- DrawTarget
-- 

local function HideTarget(self)
  if self.target then
    self.target:Hide();
  end
end

local function DrawTarget(parent, px, py)
  -- target icon

  parent.target:ClearAllPoints();
  parent.target:SetPoint("CENTER", parent, "TOPLEFT", px, py);
  parent.target:Show();
end

---------------------------------------------------------------------
-- DrawLine
--
local Line;
local LINEFACTOR = 64/126; -- Multiplying factor for texture coordinates

local function HideLine()
  if Line then Line:Hide(); end
end

local function DrawLine(C, sx, sy, ex, ey)
	-- debug("DrawLine("..sx..", "..sy..", "..ex..", "..ey..")");

	local relPoint = "TOPLEFT"

	if (not Line) then
		Line = C:CreateTexture()
		if not Line:SetTexture("Interface\\AddOns\\CruiseControl\\line.tga") then
			error("DrawLine: SetTexture failed");
		end
		Line:SetDrawLayer("OVERLAY");
		Line:SetVertexColor(LINECOLOR[1], LINECOLOR[2], LINECOLOR[3], LINECOLOR[4]);
	end

	-- Determine dimensions and center point of line
	local dx,dy = ex - sx, ey - sy;

	-- Calculate actual length of line
	local l = math.sqrt((dx * dx) + (dy * dy));
	local w = LINEWIDTH;

	-- Quick escape if it's zero length
	if l == 0 then
		Line:ClearAllPoints();
		--Line:SetTexCoord(0,0,0,0,0,0,0,0);
		--Line:SetPoint("BOTTOMLEFT", C, relPoint, cx, cy);
		--Line:SetPoint("TOPRIGHT",   C, relPoint, cx, cy);
		Line:Hide()
		return Line;
	end

	local cx,cy = (sx + ex) / 2, (sy + ey) / 2;

	-- Normalize direction if necessary
	if (dx < 0) then
		dx,dy = -dx,-dy;
	end

	-- Sin and Cosine of rotation, and combination (for later)
	local s,c = -dy / l, dx / l;
	local sc = s * c;

	-- Calculate bounding box size and texture coordinates
	local Bwid, Bhgt, BLx, BLy, TLx, TLy, TRx, TRy, BRx, BRy;
	if (dy >= 0) then
		Bwid = ((l * c) - (w * s)) * LINEFACTOR;
		Bhgt = ((w * c) - (l * s)) * LINEFACTOR;
		BLx, BLy, BRy = (w / l) * sc, s * s, (l / w) * sc;
		BRx, TLx, TLy, TRx = 1 - BLy, BLy, 1 - BRy, 1 - BLx;
		TRy = BRx;
	else
		Bwid = ((l * c) + (w * s)) * LINEFACTOR;
		Bhgt = ((w * c) + (l * s)) * LINEFACTOR;
		BLx, BLy, BRx = s * s, -(l / w) * sc, 1 + (w / l) * sc;
		BRy, TLx, TLy, TRy = BLx, 1 - BRx, 1 - BLx, 1 - BLy;
		TRx = TLy;
	end

	-- Thanks Blizzard for adding (-)10000 as a hard-cap and throwing errors!
	-- The cap was added in 3.1.0 and I think it was upped in 3.1.1
	--  (way less chance to get the error)
	--if TLx > 10000 then TLx = 10000 elseif TLx < -10000 then TLx = -10000 end
	--if TLy > 10000 then TLy = 10000 elseif TLy < -10000 then TLy = -10000 end
	--if BLx > 10000 then BLx = 10000 elseif BLx < -10000 then BLx = -10000 end
	--if BLy > 10000 then BLy = 10000 elseif BLy < -10000 then BLy = -10000 end
	--if TRx > 10000 then TRx = 10000 elseif TRx < -10000 then TRx = -10000 end
	--if TRy > 10000 then TRy = 10000 elseif TRy < -10000 then TRy = -10000 end
	--if BRx > 10000 then BRx = 10000 elseif BRx < -10000 then BRx = -10000 end
	--if BRy > 10000 then BRy = 10000 elseif BRy < -10000 then BRy = -10000 end

	-- Set texture coordinates and anchors
	Line:ClearAllPoints();
	Line:SetTexCoord( TLx, TLy, BLx, BLy, TRx, TRy, BRx, BRy )
	Line:SetPoint("BOTTOMLEFT", C, relPoint, cx - Bwid, cy - Bhgt);
	Line:SetPoint("TOPRIGHT",   C, relPoint, cx + Bwid, cy + Bhgt);
	Line:Show()
	return Line
end

----------------------------------------------------------------------
-- round coordinates to a certain precision
local coord_fmt = "%%.%df, %%.%df"
local function RoundCoords(x,y,prec)
  local fmt = coord_fmt:format(prec, prec)
  return fmt:format(x*100, y*100)
end

----------------------------------------------------------------------
-- main update function
CruiseControl:SetScript("OnUpdate", function (self, elapsed)

  self.timeSinceLastUpdate = self.timeSinceLastUpdate + elapsed;
	
  if self.timeSinceLastUpdate > UPDATE_INTERVAL then
    local C, Z, x, y = Astrolabe:GetCurrentPlayerPosition();
    local px, py = GetPlayerMapPosition("player");
    local ang = GetPlayerFacing();
    local dx, dy;
    local rx, ry;
    local mx, my = GetCurrentCursorPosition();
    local scale = WorldMapDetailFrame:GetEffectiveScale();

    local speed = GetUnitSpeed(self.currentUnit);
    if (speed and speed >= 0) then
      if (speed > MaxSpeed) then
        MaxSpeed = speed;
      end
      if (IsSwimming() or IsFlying()) then
        -- correct speed for Angle
        speed = speed * math.cos(GetUnitPitch(self.currentUnit));
      end
    else
      speed = 0;
    end

    if (
        (px ~= PlayerX or py ~= PlayerY or Angle ~= ang)
	or (mx ~= MouseX or my ~= MouseY)
	and (px ~= 0 and py ~= 0)
	) then
      -- something changed, redraw
      PlayerX = px;
      PlayerY = py;
      Angle = ang;
      MouseX = mx;
      MouseY = my;
      width = self:GetWidth();
      height = self:GetHeight();
      linelength = width + height;
      py = -py;
      rx, ry = math.sin(ang) * -1, math.cos(ang);
      px = px * width;
      py = py * height;

      local kx;
      if (rx > 0) then
      	kx = (width - px)/rx;
      else
	kx = -px/rx;
      end

      local ky;
      if (ry < 0) then
      	ky = (-height - py)/ry;
      else
      	ky = -py/ry;
      end

      local k = kx;
      if math.abs(k) > math.abs(ky) then k = ky end

      if (px == 0 and py == 0) then 
        HideLine();
	HideTarget(self);
      else
        DrawLine(self, px, py, px + rx*k, py + ry*k);
	-- draw the target destination ring
      	if(mx and my) then
      	  my = -my;
	  mx = mx * width;
	  my = my * height;
	  local oob = false;
	  local dist = (mx*rx + my*ry - rx*px - ry*py) / (rx*rx + ry*ry);
	  local qx, qy = px + dist * rx, py + dist * ry;
	  if (qx < 0 or qy > 0 or qx > width or qy < -height) then oob = true end

	  scaledFontSize = FONTSIZE / scale;
	  self.textString:SetFont(FONTSTYLE, scaledFontSize, FONTOUTLINE)

	  local yards = Astrolabe:ComputeDistance(C, Z, x, y, C, Z, MouseX, MouseY);
	  if (dist > 0 and not oob) then
	    DrawTarget(self, qx, qy);
	    yards = Astrolabe:ComputeDistance(C, Z, x, y, C, Z, qx/width, -qy/height);
	  else
	    HideTarget(self);
	  end
          if From.TOA > 0 then
	    self.textString:SetFormattedText("Speed %s: %.1f/sec {%.1f, %.1f} %d secs left  Cursor: %.1f,%.1f Dist: %d secs",
	    				     self.currentUnit, speed,
				             x*100, y*100,
                                             From.TOA,
	    			             MouseX*100, MouseY*100, yards/MaxSpeed
				            );
          else
	    self.textString:SetFormattedText("Speed %s: %.1f/sec {%.1f, %.1f}  Cursor: %.1f,%.1f Dist: %d secs",
	    				     self.currentUnit, speed,
				             x*100, y*100,
	    			             MouseX*100, MouseY*100, yards/MaxSpeed
	  			            );
          end
        else
	  HideTarget(self);
	  scaledFontSize = FONTSIZE / scale;
	  self.textString:SetFont(FONTSTYLE, scaledFontSize, FONTOUTLINE);
	  if From.TOA > 0 then
	    self.textString:SetFormattedText("Speed %s: %.1f/sec {%.1f, %.1f} %d secs left",
	  			             self.currentUnit, speed,
				             x*100, y*100, From.TOA
				            );
          else
	    self.textString:SetFormattedText("Speed %s: %.1f/sec {%.1f, %.1f}",
	  			             self.currentUnit, speed,
				             x*100, y*100
				            );
          end
	end
      end
    end
    
    self.timeSinceLastUpdate = 0;
  end
end);

----------------------------------------------------------------------
-- listener for vehicle change
-- register events

function CruiseControl:ADDON_LOADED(addon)
  if addon ~= "CruiseControl" then
    return;
  end
  self:UnregisterEvent("ADDON_LOADED");
  self:Init();
end

function CruiseControl:UNIT_ENTERED_VEHICLE(who)
  if who == "player" then
    CruiseControl.currentUnit = "vehicle";
  end
end

function CruiseControl:UNIT_EXITED_VEHICLE(who)
  if who == "player" then
    CruiseControl.currentUnit = "player";
  end
end

CruiseControl:SetScript("OnEvent", function(self, event, ...)
  self[event](self, ...);
end);

CruiseControl:RegisterEvent("UNIT_ENTERED_VEHICLE")
CruiseControl:RegisterEvent("UNIT_EXITED_VEHICLE")
CruiseControl:RegisterEvent("ADDON_LOADED")

